﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Shell32;
using System.IO;

namespace CKMusicPlayer
{
    public partial class F14_AddFile : Form
    {
        /// <summary> [コンストラクター] </summary>
        public F14_AddFile()
        {
            InitializeComponent();

        }

        /// <summary> [ListMF]プロパティ(読取り専用) </summary>
        public List<CMusicFile> ListMF
        {
            get { return _lstMF; }
        }

        const string CK_OF = "□";
        const string CK_ON = "■";
        const string CLM_CHK = "Check";

        private List<CMusicFile> _lstMF = new List<CMusicFile>();

        /// <summary> [全解除]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDisSelectAll_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < grdView.Rows.Count; i++)
            {
                grdView[0, i].Value = CK_OF;
            }
            nudSelect.Value = 0;
            btnSelectAll.Enabled = true;
        }

        /// <summary> [参照]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetAlbum_Click(object sender, EventArgs e)
        {
            const string DEF_FOLDER = @"C:\";
            string iniFolder = Properties.Settings.Default.F14_OpenFolder;
            if (iniFolder == DEF_FOLDER) iniFolder = Properties.Settings.Default.AlbumFolder;
            if (!Directory.Exists(iniFolder)) iniFolder = DEF_FOLDER;

            if (!CC.IsWin7 || CC.UseFolderDlg)
            {
                folderDlg.Description = "フォルダを選択してください。";
                folderDlg.ShowNewFolderButton = false;
                folderDlg.SelectedPath = iniFolder;

                if (folderDlg.ShowDialog() == CC.RSLT_OK)
                {
                    GetAlbum(folderDlg.SelectedPath);
                }
            }
            else
            {
                using (var fcom = new Fcom_FolderDlg())
                {
                    fcom.SelectedPath = iniFolder;
                    if (fcom.ShowDialog() == CC.RSLT_OK)
                    {
                        GetAlbum(fcom.SelectedPath);
                    }
                }
            }
        }

        /// <summary> [OK]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            _lstMF.Clear();

            for (int i = 0; i < grdView.Rows.Count; i++)
            {
                string chk = grdView[0, i].Value.ToString();
                if (chk == CK_ON)
                {
                    string mfile = grdView[5, i].Value.ToString();
                    string title = grdView[1, i].Value.ToString();
                    if (rdoCustom.Checked && !File.Exists(mfile))
                    {
                        string msg = "「" + title + "」用の次のファイルが見つかりません。"
                                + CC.NL + CC.NL + "「" + mfile + "」";
                        CC.ShowMsg(msg, CC.EXC);
                        return;
                    }
                    var cmf = new CMusicFile();
                    cmf.Title = title;
                    cmf.Artist = grdView[2, i].Value.ToString();
                    cmf.Length = grdView[3, i].Value.ToString();
                    cmf.VRatio = Convert.ToDouble(grdView[4, i].Value);
                    cmf.File = mfile;
                    _lstMF.Add(cmf);
                }
            }
            if (_lstMF.Count > 0)
            {
                this.DialogResult = CC.RSLT_OK;
                this.Close();
            }
            else    // 通常、この事態は発生しない
                CC.ShowMsg("チェックされたデータが存在しません。", CC.EXC);

        }

        /// <summary> [全選択]ボタンクリックイベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSelectAll_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < grdView.Rows.Count; i++)
            {
                grdView[0, i].Value = CK_ON;
            }
            nudSelect.Value = grdView.Rows.Count;
        }

        /// <summary> [cbxCustom_SIChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbxCustom_SIChanged(object sender, EventArgs e)
        {
            if (cbxCustom.SelectedIndex < 0) return;

            string customlist = cbxCustom.Text;
            string filefp = CC.CUST_FOLD + @"\" + cbxCustom.Text + ".txt";
            if (!File.Exists(filefp))      // 通常、この事態は発生しない
            {
                cbxCustom.BackColor = Color.Red;
                CC.ShowMsg("このリストは存在しません。", CC.EXC);
                cbxCustom.BackColor = CC.COL_WIN;
                cbxCustom.SelectedIndex = -1;
            }
            else
                ReadList(filefp);
        }

        /// <summary> [Form_Load]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Load(object sender, EventArgs e)
        {
            //CC.SetForm(this);
            this.StartPosition = FormStartPosition.CenterParent;
            InitializeForm();
        }

        /// <summary> [grdView_CellClick]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grdView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (grdView.SelectedRows.Count == 0) return;     // Ver1.1.0で追加
            
            int idx = grdView.SelectedRows[0].Index;
            grdView.ClearSelection();
            string chk = grdView[0, idx].Value.ToString();

            if (chk == CK_ON)
            {
                grdView[0, idx].Value = CK_OF;
                nudSelect.Value -= 1;
                btnSelectAll.Enabled = true;
            }
            else
            {
                grdView[0, idx].Value = CK_ON;
                nudSelect.Value += 1;
                btnOK.Enabled = true;
                btnDisSelectAll.Enabled = true;
            }
            SelectNextControl(this.ActiveControl, true, true, true, true);
        }

        /// <summary> [nudSelect_ValueChanged]イベント </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void nudSelect_ValueChanged(object sender, EventArgs e)
        {
            int gcount = grdView.Rows.Count;
            if (gcount > 0)
            {
                if (nudSelect.Value == 0)
                {
                    btnDisSelectAll.Enabled = false;
                    btnOK.Enabled = false;
                    btnSelectAll.Enabled = true;
                }
                else if (nudSelect.Value == gcount)
                {
                    btnDisSelectAll.Enabled = true;
                    btnOK.Enabled = true;
                    btnSelectAll.Enabled = false;
                }
                else
                {
                    btnDisSelectAll.Enabled = true;
                    btnOK.Enabled = true;
                    btnSelectAll.Enabled = true;
                }
            }
            else
            {
                    btnDisSelectAll.Enabled = false;
                    btnOK.Enabled = false;
                    btnSelectAll.Enabled = false;
            }
        }

        /// <summary> [Rdo_CheckedChanged]イベント
        /// [rdoAlbum],[rdoCustom]に対して適用
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Rdo_CheckedChanged(object sender, EventArgs e)
        {
            var rbtn = (RadioButton)sender;
            if (!rbtn.Checked) return;
            grdView.Rows.Clear();
            nudSelect.Value = 0;
            btnDisSelectAll.Enabled = false;
            btnOK.Enabled = false;
            btnSelectAll.Enabled = false;

            if (rbtn == rdoAlbum)
            {
                btnGetAlbum.Enabled = true;
                txtAlbum.BackColor = CC.COL_WIN;
                cbxCustom.SelectedIndex = -1;
                cbxCustom.Enabled = false;
            }
            else
            {
                btnGetAlbum.Enabled = false;
                txtAlbum.BackColor = CC.COL_CTR;
                txtAlbum.Text = String.Empty;
                cbxCustom.Enabled = true;
            }
        }


        /// <summary> [IsExistSong]メソッド
        /// [GetAlbum]から呼出し
        /// </summary>
        /// <param name="vfp">フォルダー名(フルパス)</param>
        /// <param name="vlstSongs"></param>
        /// <returns></returns>
        private bool IsExistSong(string vfp, ref List<string> vlstSongs)
        {
            if (vfp.Length == 0) return false;

            vlstSongs = new List<string>();
            foreach (string sfile in Directory.GetFiles(vfp))
            {
                for (int i = 0; i < CC.EXTS.Length; i++)
                {
                    if (Path.GetExtension(sfile).ToLower() == CC.EXTS[i])
                    {
                        vlstSongs.Add(sfile);
                        break;
                    }
                }
            }
            return (vlstSongs.Count > 0);
        }

        /// <summary> [FillCbxCustom]メソッド
        /// [InitializeForm]から呼出し
        /// </summary>
        private void FillCbxCustom()
        {
            cbxCustom.Items.Clear();
            if (!Directory.Exists(CC.CUST_FOLD)) return;

            foreach (string tfile in Directory.GetFiles(CC.CUST_FOLD))
            {
                if (Path.GetExtension(tfile).ToLower() == CC.EXT_TXT)
                    cbxCustom.Items.Add(Path.GetFileNameWithoutExtension(tfile));
            }
        }

        /// <summary> [GetAlbum]メソッド
        /// [btnGetAlbum_Click]から呼出し
        /// </summary>
        /// <param name="vfolderFP"></param>
        private void GetAlbum(string vfolderFP)
        {
            grdView.Rows.Clear();
            btnDisSelectAll.Enabled = false;
            btnOK.Enabled = false;

            var lstSongs = new List<string>();
            if (IsExistSong(vfolderFP, ref lstSongs))
            {
                PutData2GrdView(lstSongs);
                if (grdView.Rows.Count > 0) btnSelectAll.Enabled = true;

                grdView.ClearSelection();
                txtAlbum.Text = vfolderFP.Substring(vfolderFP.LastIndexOf(@"\") + 1);
                Properties.Settings.Default.F14_OpenFolder = vfolderFP;
                Properties.Settings.Default.Save();
            }
            else
            {
                txtAlbum.Text = String.Empty;
                btnSelectAll.Enabled = false;
            }
            nudSelect.Value = 0;
        }

        /// <summary> [InitializeForm]メソッド
        /// [Form_Load]から呼出し
        /// </summary>
        private void InitializeForm()
        {
            nudSelect.Visible = false;
            txtAlbum.ReadOnly = true;
            txtAlbum.TabStop = false;
            rdoAlbum.Checked = true;
            FillCbxCustom();
            rdoCustom.Enabled = (cbxCustom.Items.Count > 0);
            btnOK.Enabled = false;
            btnSelectAll.Enabled = false;
            btnDisSelectAll.Enabled = false;

            SetGrdView();
        }

        /// <summary> [PutData2GrdView]メソッド
        /// [GetAlbum]から呼出し
        /// </summary>
        private void PutData2GrdView(List<string> vlstSongs)
        {
            for (int i1 = 0; i1 < vlstSongs.Count; i1++)
            {
                string fp = vlstSongs[i1];
                string folder = Path.GetDirectoryName(fp);
                string file = Path.GetFileName(fp);

                grdView.Rows.Add();
                DataGridViewRow grow = grdView.Rows[grdView.Rows.Count - 1];
                grow.Cells[CLM_CHK].Value = CK_OF;
                grow.Cells[CC.CLM_FILE].Value = fp;
                grow.Cells[CC.CLM_VRATIO].Value = 1.0;

                var shell = new ShellClass();
                Folder f = shell.NameSpace(folder);
                FolderItem item = f.ParseName(file);

                int count = 0;
                for (int i2 = 0; i2 < 300; i2++)
                {
                    string tname = f.GetDetailsOf(null, i2);
                    if (tname == CC.CLM_TITLE)
                    {
                        grow.Cells[tname].Value = f.GetDetailsOf(item, i2);
                        count += 1;
                    }
                    else if (tname == CC.ClmArtist)
                    {
                        grow.Cells[CC.CLM_ART_XP].Value = f.GetDetailsOf(item, i2);
                        count += 1;
                    }
                    else if (tname == CC.CLM_LENGTH)
                    {
                        grow.Cells[tname].Value = f.GetDetailsOf(item, i2);
                        count += 1;
                    }
                    if (count >= 3) break;
                }
            }
        }

        /// <summary> [ReadList]メソッド
        /// [cbxCustom_SIChanged]から呼出し
        /// </summary>
        /// <param name="vfp"></param>
        private void ReadList(string vfp)
        {
            grdView.Rows.Clear();
            btnDisSelectAll.Enabled = false;
            btnOK.Enabled = false;

            using (var reader = new StreamReader(vfp, Encoding.Default))
            {
                int count = 0;
                try
                {
                    while (true)
                    {
                        string sline = reader.ReadLine();
                        if (sline == null) break;

                        string[] separator = { CC.DLMTR };
                        string[] data = sline.Split(separator, StringSplitOptions.None);

                        grdView.Rows.Add();
                        grdView[0, count].Value = CK_OF;
                        grdView[1, count].Value = data[0];
                        grdView[2, count].Value = data[1];
                        grdView[3, count].Value = data[2];
                        grdView[4, count].Value = Convert.ToDecimal(data[3]);
                        grdView[5, count].Value = data[4];
                        count += 1;                        
                    }
                    if (count > 0) btnSelectAll.Enabled = true;
                }
                catch (Exception ex)
                {
                    CC.ShowMsg(ex.Message, CC.EXC);
                    grdView.Rows.Clear();
                }
                finally
                {
                    reader.Close();
                    nudSelect.Value = 0;
                }
            }
            grdView.ClearSelection();
        }

        /// <summary> [SetGrdView]：grdViewの初期状態を設定する
        /// [InitializeForm]から呼出し
        /// </summary>
        private void SetGrdView()
        {
            grdView.Columns.Add(CLM_CHK, String.Empty);
            grdView.Columns.Add(CC.CLM_TITLE, CC.CLM_TITLE);
            grdView.Columns.Add(CC.CLM_ART_XP, CC.CLM_ART_XP);
            grdView.Columns.Add(CC.CLM_LENGTH, CC.CLM_LENGTH);
            grdView.Columns.Add(CC.CLM_VRATIO, CC.CLM_VRATIO);
            grdView.Columns.Add(CC.CLM_FILE, CC.CLM_FILE);

            grdView.Columns[CLM_CHK].Width = 26;
            grdView.Columns[CC.CLM_TITLE].Width = 294;
            grdView.Columns[CC.CLM_ART_XP].Width = 120;
            grdView.Columns[CC.CLM_LENGTH].Width = 85;
            grdView.Columns[CC.CLM_VRATIO].Visible = false;
            grdView.Columns[CC.CLM_FILE].Visible = false;

            CC.SetGridProperty(grdView);
            grdView.AllowUserToResizeColumns = true;
        }


    }
}
